@@ -2,6 +2,8 @@
 #'
 #' @description This function generates a simulated multivariate VAR time series.
 #' 
+#' @usage simulateVAR(N, p, nobs, rho, sparsity, method, covariance)
+#' 
 #' @param N dimension of the time series. 
 #' @param p number of lags of the VAR model.
 #' @param nobs number of observations to be generated.
@@ -19,11 +21,10 @@
 #'
 #' @export
 simulateVAR <- function(N = 100, p = 1, nobs = 250, rho = 0.5, sparsity = 0.05, 
-                        method = "normal", covariance = "toeplitz") {
+                        method = "normal", covariance = "Toeplitz") {
   
   # Create the list of the VAR matrices
   A <- list()
-  cA <- matrix(0, nrow = N, ncol = N * p)
   for (i in 1:p) {
     A[[i]] <- createSparseMatrix(sparsity = sparsity, N = N, method = method, stationary = TRUE, p = p)
     l <- max(Mod(eigen(A[[i]])$values))
@@ -32,7 +33,6 @@ simulateVAR <- function(N = 100, p = 1, nobs = 250, rho = 0.5, sparsity = 0.05,
       l <- max(Mod(eigen(A[[i]])$values))
     }
     A[[i]] <- 1/sqrt(p) * A[[i]]
-    cA[1:N, ((i-1) * N) + (1:N)] <- A[[i]]
   }
 
   # Covariance Matrix: Toeplitz, Block1 or Block2
@@ -53,15 +53,22 @@ simulateVAR <- function(N = 100, p = 1, nobs = 250, rho = 0.5, sparsity = 0.05,
     r[(l+1):N, (l+1):N] <- rho
     T <- I + r
       
-  } else if (covariance == "toeplitz"){
+  } else if (covariance == "Toeplitz"){
     
     r <- rho^(1:N)
     T <- Matrix::toeplitz(r) 
   
+  } else if (covariance == "Wishart"){
+
+    r <- rho^(1:N)
+    S <- Matrix::toeplitz(r)
+    T <- stats::rWishart(1, 2*N, S)
+    T <- as.matrix(T[, , 1])
+
   } else if (covariance == "diagonal"){
     
     T <- diag(x = rho, nrow = N, ncol = N)
-
+    
   } else {
     
     stop("Unknown covariance matrix type. Possible choices are: toeplitz, block1, block2 or diagonal")
@@ -70,16 +77,60 @@ simulateVAR <- function(N = 100, p = 1, nobs = 250, rho = 0.5, sparsity = 0.05,
   
   # Matrix for MA part
   theta <- matrix(0, N, N)
-  ar <- 1:p
+  # ar <- 1:p
   
   # Generate the VAR process 
-  data <- MTS::VARMAsim(nobs = nobs, arlags = ar, malags = 0, cnst = 0, phi = cA, theta = theta, skip = 200, sigma = T)
+  data <- generateVARseries(nobs = nobs, mu = 0, AR = A, sigma = T, skip = 200)
   
   # Output
   out <- list()
-  out$data <- data
   out$A <- A
-  out$S <- T
+  out$series <- data$series
+  out$noises <- data$noises
+  out$sigma <- T
+  
+  attr(out, "class") <- "var"
+  attr(out, "type") <- "simulation"
   return(out)
   
 }
+
+generateVARseries <- function(nobs, mu, AR, sigma, skip = 200) {
+
+  ## This function creates the simulated time series 
+  
+  N <- nrow(sigma)
+  nT <- nobs + skip
+  at <- mvtnorm::rmvnorm(nT, rep(0,N), sigma)
+  
+  p <- length(AR)
+
+  ist <- p + 1
+  zt <- matrix(0, nT, N)
+  
+  if(length(mu)==0) {
+    mu <- rep(0,N)
+  }
+  
+  for (it in ist:nT){
+    tmp <- matrix(at[it,], 1, N)
+
+    for (i in 1:p){
+      ph <- AR[[i]]
+      ztm <- matrix(zt[it-i, ], 1, N)
+      tmp <- tmp + ztm%*%t(ph)
+    }
+
+    zt[it, ] <- mu + tmp
+  }
+  
+  # skip the first skip points to initialize the series
+  zt <- zt[(1+skip):nT, ]
+  at <- at[(1+skip):nT, ]
+  
+  out <- list()
+  out$series <- zt
+  out$noises <- at
+  return(out)
+  
+}
\ No newline at end of file
